require 'recursive_open_struct'
require 'json'

require 'dnsblim/cli/tables'

module Dnsblim
  # A class representing a reply from the API
  class APIResponse
    attr_accessor :reply, :success

    def inspect
      string = "#<#{self.class.name}:#{object_id} "
      string << @reply.inspect << '>'
    end

    def to_s
      @reply
    end
    
    # @param [HTTParty::Response] api_reply Response received by the API
    # @param [Integer] api_status HTTP Status Code for response
    # @param [Hash] args A hash of options to parse with
    def initialize(api_reply, api_status, **args)
      if args.fetch(:ip_addr, nil)
        @ip_addr = args.fetch(:ip_addr)
      end
      if args.fetch(:quiet, nil)
        @quiet = args.fetch(:quiet)
      end
      @reply = JSON.parse(api_reply)
      if @reply.fetch('reply', nil)
        if @reply['reply'].fetch('mysql', nil)
          @mysql = @reply['reply'].fetch('mysql', nil)
        end
        if @reply['reply'].fetch('bl', nil)
          @bl = @reply['reply'].fetch('bl', nil)
        end
      end
      if @reply.fetch('list', nil) and @reply.fetch('prune', nil)
        @record_type = args[:record_type]
      end
      if @reply.fetch('dupes', nil) and @reply.fetch('message', nil)
        @ips = args.fetch(:ip_addrs, nil)
        @duplicates = @reply['dupes']
        @message = @reply['message']
      end
      @status_code = api_status
      @success = @reply['success']
    end

    # @param [HTTParty::Response] api_reply Response received by the API
    # @param [Integer] api_status HTTP Status Code for response
    # @param [Hash] args A hash of options to parse with
    # @return [APIResponse.new()]
    def self.parse(api_reply, api_status, **args)
      new(api_reply, api_status, args)
    end

    #
    def success?
      @success
    end

    # @param [String] type Type of response from API
    def print_table(type)
      case type
      when 'stats'
        Dnsblim::CLI::Tables::StatsTable.new.run(@reply, {quiet: @quiet, record_type: @record_type})
      when 'check'
        Dnsblim::CLI::Tables::CheckTable.new.run(@reply, {quiet: @quiet, ip_addr: @ip_addr})
      when 'mcheck'
        Dnsblim::CLI::Tables::MCheckTable.new.run(@reply, {quiet: @quiet})
      when 'madd'
        Dnsblim::CLI::Tables::AdditionTable.new.run(@reply, {quiet: @quiet, additions: 'multiple', message: @message, success: @success, duplicates: @duplicates, ip_addrs: @ips})
      when 'add'
        Dnsblim::CLI::Tables::AdditionTable.new.run(@reply, {quiet: @quiet, additions: 'single', message: @message, success: @success, duplicates: @duplicates, ip_addrs: @ips})
      else
        raise ArgumentError('Invalid Type given to Dnsblim::APIResponse.print_table')
      end
    end
  end
end