require 'dnsblim/methods'
require 'dnsblim/desc'
require 'thor'
require 'thor/actions'
# rubocop:disable Layout/AlignParameters
module Dnsblim
  # @note Thor based class for command line interface
  class App < Thor
    include Thor::Actions
    class_option(:config,
        type: :string,
        default: '~/.dnsblimrc',
        aliases: %w[-c],
        desc: 'config file to pull settings from')

    class_option(:quiet,
        type: :boolean,
        default: false,
        aliases: %w[-q],
        desc: 'quiets some of the output')

    method_option(:table, default: true, type: :boolean, aliases: %w[-t], desc: 'If false, gives the raw API response')
    desc 'add [options] <IP> <TYPE> <REASON...>', 'Add an IP into the blacklist'
    long_desc Dnsblim::Docs::Desc::ADD
    def add(ip_addr, ltype, reason)
      methods = Dnsblim::Methods.new(options)
      methods.add(ltype, ip_addr, reason)
    end

    method_option(:table, default: true, type: :boolean, aliases: %[-t], desc: 'If false, gives the raw API response')
    method_option(:type, required: true, type: :string, desc: 'Type of listing 1,2,3,4,5,6,7 See https://dnsbl.im/use')
    method_option(:reason, required: true, type: :string, desc: 'Reason for listing')
    desc 'fadd ', 'Submit IPs via a file'
    def fadd(file)
      methods = Dnsblim::Methods.new(options)
      methods.fadd(file, type: options[:type], reason: options[:reason], quiet: options[:quiet])
    end

    method_option(:table, default: true, type: :boolean, aliases: %w[-t], desc: 'If false, gives the raw API response')
    desc 'madd [options] \'TYPE\' \'REASON...\' [IPs ...]', 'Add IPs into the blacklist'
    long_desc Dnsblim::Docs::Desc::MADD
    def madd(type, reason, *ips)
      methods = Dnsblim::Methods.new(options)
      methods.madd(type, reason, ips)
    end

    method_option(:table, default: true, type: :boolean, aliases: %w[-t], desc: 'If false, gives the raw API response')
    desc 'check [options] [input...]', 'Check an IP against the blacklist'
    long_desc Dnsblim::Docs::Desc::CHECK
    def check(ip_addr)
      methods = Dnsblim::Methods.new(options)
      methods.check(ip_addr)
    end

    desc 'mcheck [options] [input...]', 'Check IPs against the blacklist'
    method_option(:table, default: true, type: :boolean, aliases: %w[-t], desc: 'If false, gives the raw API response')
    long_desc Dnsblim::Docs::Desc::MCHECK
    def mcheck(*ip_addrs)
      methods = Dnsblim::Methods.new(options)
      methods.mcheck(ip_addrs)
    end


    desc 'stats', 'Return stats from the database.'
    method_option(:table, default: true, type: :boolean, aliases: %w[-t], desc: 'If false, gives the raw API response')
    long_desc Dnsblim::Docs::Desc::STATS
    def stats
      methods = Dnsblim::Methods.new(options)
      methods.stats(options[:record_type])
    end

    desc 'config', 'Return config files for services.'
    method_option(:software, type: :string, required: true, aliases: %w[-t -s], desc: 'selects the software type/name to return')
    long_desc Dnsblim::Docs::Desc::CONFIG
    def config
      methods = Dnsblim::Methods.new(options)
      methods.config(options[:software])
    end

    desc 'init', 'Initialize the gem for first run'
    long_desc Dnsblim::Docs::Desc::INIT
    def init
      methods = Dnsblim::Methods.new(options)
      methods.init(options)
    end
  end
end
# rubocop:enable Layout/AlignParameters