require 'command_line_reporter'
require 'highline'
HighLine.colorize_strings

require 'paint'
module CommandLineReporter
  class NestedFormatter
    def format(options, block)
      validate_options(options, *VALID_OPTIONS)

      indent_level :incr

      padding = ' ' * @indent_level * (options[:indent_size] || indent_size)

      message_str = padding + (options[:message] || message_string)
      complete_str = options[:complete] || complete_string

      if options[:type] == 'inline'
        colorize(message_str.to_s, true, options)
      else
        colorize(message_str, false, options)
        complete_str = padding + complete_str
      end

      block.call

      colorize(complete_str, false, options)

      indent_level :decr
    end
  end
end

module Dnsblim
  module CLI
    module Tables
      class MCheckTable
        include ::CommandLineReporter

        def initialize
          @hl = HighLine.new(STDIN, STDOUT, 80, 1, 2, 1)
          self.formatter = 'nested'
        end

        def run(obj, _options)
          obj.each do |ip, hash|
            report(message: "Check on #{ip}") do
              report(message: "MySQL Data for #{ip}", color: 'green') do
                hash[1]['mysql'].each do |_key, _value|
                  case _key
                  when 'success'
                    return_string = "Retrieval Success: #{_value.to_s.capitalize}"
                    report(message: return_string, type: 'inline', complete: '') do
                    end
                  when 'ip_addr'
                    next
                  when 'type'
                    return_string = "Listing Type: #{_value}"
                    report(message: return_string, type: 'inline', complete: '') do
                    end
                  when 'removal_date'
                    return_string = "Expiration Date: #{DateTime.parse(_value).strftime('%c')}"
                    report(message: return_string, type: 'inline', complete: '') do
                    end
                  when 'class'
                    return_string = "Type Class: #{_value}"
                    report(message: return_string, type: 'inline', complete: '') do
                    end
                  when 'error'
                    return_string = "Retrieved: #{_value}"
                    report(message: return_string, type: 'inline', color: 'red', complete: '') do
                    end
                  end
                end
              end
              report(message: "DNS Results for #{ip}", color: 'green') do
                hash[1]['dns'].each do |_key, _value|
                  return_string = ''
                  case _key
                  when 'success'
                    case _value

                    when true
                      return_string = "Retrieved: #{_value.to_s.capitalize}"
                      report(message: return_string, type: 'inline', complete: '') do
                      end
                    when false
                      return_string = 'Retrieved: Not listed'
                      report(message: return_string, type: 'inline', complete: '', color: 'red') do
                      end
                    end
                  when 'ip_addr'
                    next
                  when 'reply'
                    return_string = "Record: #{_value}"
                    report(message: return_string, type: 'inline', complete: '') do
                    end
                  when 'listed'
                    return_string = "Listed: #{_value == 1 ? 'Yes' : 'No'}"
                    report(message: return_string, type: 'inline', complete: '') do
                    end

                  end
                end
              end

              hash[1].each do |_key, _value|
                return_string = ''
                case _key
                when 'mysql'
                  next
                when 'dns'
                  next
                when 'set_by'
                  return_string = "Submitted By: #{_value}"
                when 'reason'
                  return_string = "Reason: #{_value}"
                when 'set_by_id'
                  return_string = "Key ID: #{_value}"
                end
                report(message: return_string, type: 'inline', complete: '') do
                end
              end
            end
          end
        end
      end

      class StatsTable
        include ::CommandLineReporter

        def initialize
          @hl = HighLine.new(STDIN, STDOUT, 80, 1, 2, 1)
          # self.formatter = 'nested'
        end

        def run(obj, _options)
          table(border: true, width: 65) do
            row(header: true) do
              column('Metric', width: 25, align: 'right')
              column('Value')
            end
            obj.each do |key, value|
              case key
              when 'list'
                value.each do |keyd, val|
                  row do
                    case keyd
                    when 'count'
                      column('Total')
                    when '1'
                      column('Tor | 1')
                    when '2'
                      column('HTTP(S) Proxies | 2')
                    when '3'
                      column('SOCKS(4,5) | 3')
                    when '4'
                      column('Open Router | 4')
                    when '5'
                      column('Abusive Hosts | 5')
                    when '6'
                      column('Current Events | 6')
                    when '7'
                      column('MTA/Mail Servers | 7')
                    when 'test'
                      column('Test Classes | 250-255')
                    end
                    column(val.to_s)
                  end
                end
              when 'prune'
                row do
                  column 'Expiring', align: 'right'
                  column value.to_s
                end
              when 'fail2ban', 'irc2ban'
                row do
                  column key.to_s.upcase.to_s
                  column value.to_s
                end
              end
            end
          end
        end
      end

      class CheckTable
        include ::CommandLineReporter

        def initialize
          @hl = HighLine.new(STDIN, STDOUT, 80, 1, 2, 1)
          self.formatter = 'nested'
        end

        def run(obj, options)
          report(message: "Check on #{options[:ip_addr]}") do
            report(message: "MySQL Data for #{options[:ip_addr]}", color: 'green') do
              obj['reply']['mysql'].each do |key, value|
                return_string = ''
                case key
                when 'success'
                  return_string = "Retrieval Success: #{value.to_s.capitalize}"
                when 'ip_addr'
                  next
                when 'type'
                  return_string = "Listing Type: #{value}"
                when 'removal_date'
                  return_string = "Expiration Date: #{DateTime.parse(value).strftime('%c')}"
                when '_class'
                  return_string = "Type Class: #{value}"
                end
                report(message: return_string, type: 'inline', complete: "\t ... retrieved") do
                end
              end
            end
            report(message: "DNS Results for #{options[:ip_addr]}", color: 'green') do
              obj['reply']['dns'].each do |key, value|
                return_string = ''
                case key
                when 'success'
                  return_string = "Retrieval Success: #{value.to_s.capitalize}"
                when 'ip_addr'
                  next
                when 'reply'
                  return_string = "Record: #{value}"
                when 'listed'
                  return_string = "Listed: #{value == 1 ? 'Yes' : 'No'}"
                end
                report(message: return_string, type: 'inline', complete: "\t ... retrieved") do
                end
              end
            end

            obj['reply'].each do |key, value|
              return_string = ''
              case key
              when 'mysql'
                next
              when 'dns'
                next
              when 'set_by'
                return_string = "Submitted By: #{value}"
              when 'reason'
                return_string = "Reason: #{value}"
              when 'set_by_id'
                return_string = "Key ID: #{value}"
              end
              report(message: return_string, type: 'inline', complete: "\t ... retrieved") do
              end
            end
          end
        end
      end
      class AdditionTable
        def initialize
          @hl = HighLine.new(STDIN, STDOUT, 120, 20, 2, 0)
          HighLine.use_color = true
        end

        def run(obj, options)
          if options[:quiet]
            STDERR.puts "Suppressing output due to --quiet"
          else
            @hl.say("<%= Paint['Contacting API', 'green'] %>")
            case options[:additions]
            when 'single'
              @hl.indent(1, "<%= Paint['Attempting to add #{options[:ip_addrs].join('')} ...', 'orange'] %>")
              if options[:duplicates].include? options[:ip_addrs].first
                @hl.indent(2, "<%= Paint['Error', 'red'] %>: #{options[:ip_addrs].join('')} already in database.")
              else
                @hl.indent(2, "<%= Paint['Added #{options[:ip_addrs].join('')}', :green] %>")
              end
              @hl.indent(1, "<%= Paint['complete', 'orange'] %>")
            when 'multiple'
              options[:ip_addrs].each do |ip|
                @hl.indent(1, "<%= Paint['Attempting to add #{ip} ...', 'orange'] %>")
                if options[:duplicates].include? ip
                  @hl.indent(2, "<%= Paint['Error', 'red'] %>: #{ip} already in database.")

                elsif !options[:duplicates].include? ip
                  @hl.indent(2, "<%= Paint['Added #{ip}', :green] %>")
                end
                @hl.indent(1, "<%= Paint['complete', 'orange'] %>")
              end
            else
              raise ArgumentError(
                <<-RAISE.squish
              case block in Dnsblim::CLI::Tables::AdditionTable#run has received something
              other than 'single' or 'multiple'
              THIS IS A BUG
            RAISE
              )

            end

            if options[:message] == %w[failure duplicates]
              @hl.say("<%= Paint['failure', 'red'] %>.. <%= Paint['ALL IPs WERE DUPLICATES', 'red'] %>")
            elsif options[:message] == %w[success duplicates]
              @hl.say("<%= Paint['complete', 'green'] %>.. <%= Paint['with duplicates', 'orange'] %>:")
              @hl.say("Duplicates: #{options[:ip_addrs].join(', ')}")
            elsif options[:message] == %w[success]
              @hl.say("<%= Paint['complete', 'green'] %>")
            end
          end
        end
      end
    end
  end
end
