module Dnsblim
  module Docs
    # Module holding command function long descriptions
    module Desc
      module_function

      CHECK = ERB.new(<<~CHECK
        'check', checks our database and DNS for information 
        on a single IP.
        --<% Paint['EXAMPLE', 'white', :bold] %>:
        ----$ dnsblim check 127.0.0.255
      CHECK
      ).result

      MCHECK = ERB.new(<<~MCHECK
        'mcheck', checks our database and DNS for information
        on a set of IPs, given at the command line.
        --<% Paint['EXAMPLE', 'white', :bold] %>:
        ----$ dnsblim mcheck 127.0.0.255 127.0.0.254 127.0.0.253....
        ----Note: 'mcheck' is limited to 10 IPs per usage
      MCHECK
      ).result

      ADD = ERB.new(<<~ADD
        'add', adds an IP through the API single-y
        This is as opposed to 'madd' which allows listing off
        #%IPs and having them all entered into the blacklist
        #*----------------------
        --<%= Paint['EXAMPLES', 'white', :bold] %>:

        ----$ dnsblim add 127.0.0.255 255 "Some good reason"
      ADD
      ).result

      MADD = ERB.new(<<~MADD
        'madd', adds multiple IPs through the API,
        This is as opposed to 'add', which only does
         1 IP per call of the command
        #*-------------------
        --<%= Paint['EXAMPLES', 'white', :bold] %>:

        ----$ dnsblim madd 255 'Testing' 127.0.0.255 127.0.0.254 127.0.0.253
        #%    127.0.0.252 127.0.0.251 127.0.0.250
      MADD
      ).result

      CONFIG = ERB.new(<<~CONFIG
        'config' returns software configurations for using DNSBL.im
        --Valid Options are:
        ----* insp, inspircd     -> InspIRCd
        ----* unreal, unrealircd -> UnrealIRCd
        ----* chary, charybdis   -> Charybdis IRCd
        ----* hopm, bopm         -> (Hybrid/Blitzd) Open Proxy Monitor
      CONFIG
      ).result
      STATS = ERB.new(<<~STATS
        'stats' returns... stats from the BlackList,
        basically how many of each listing/record type we have.

        --<%= Paint['EXAMPLE', 'white', :bold] %>:
        ----$ dnsblim stats
      STATS
      ).result

      INIT = ERB.new(<<~INIT
        'init' initializes the gem, generating a config file 
        in '#{File.expand_path('~/.dnsblimrc')}' to house the options.
        #%
        Currently, two options are housed in this file:
        #%
        --* API Key (key =)
        ----> Your DNSBL.im API Key, which authenticates
        ----   you with the API
        #%
        --* DNSBL.im API Host (api_host =)
        ----> This option allows switching the URL of where the
        ----   gem runs its requests,
        ----> If you haven't been told to,
        ----    then theres no reason to change the `api_host`

      INIT
      ).result
    end
  end
end